import argparse
import json

from .track import Track


def _get_args():
    parser = argparse.ArgumentParser(description='Take an input file and convert it to MIDI')
    parser.add_argument('infile')
    parser.add_argument(
        '-o',
        '--outfile',
        help='Name of the output file to create',
        default='out.mid',
    )
    parser.add_argument(
        '-m',
        '--mapping',
        help='Name of the mapping file produced by tasmidi-map',
        required=True,
    )
    parser.add_argument(
        '-p',
        '--program',
        type=int,
        default=4,
        help='MIDI program in your soundfont to use. Has no effect if the output is running running in a DAW.'
    )
    return parser.parse_args()


def main():
    args = _get_args()
    config = {}

    with open(args.mapping) as f:
        config.update(json.load(f))

    track = Track(args.outfile, config, args.program)

    with open(args.infile) as infile:
        for line in infile:
            track.handle(line)

    track.save()



import argparse
import json

from .notes import generate_scale

def _get_args():
    parser = argparse.ArgumentParser(
        description='Take a LibTAS input file and generate a mapping for it',
    )
    parser.add_argument('input_file', help='LibTAS input file (usually named inputs)')
    parser.add_argument(
        '-m',
        '--minor',
        help='Change to a minor scale',
        action='store_true',
    )
    parser.add_argument(
        '-t',
        '--transpose',
        help='Transpose key a number of half steps',
        type=int,
        default=0,
    )
    parser.add_argument(
        '-i',
        '--interpolate',
        help='Interpolate the scale degrees for harmony',
        action='store_true',
    )
    parser.add_argument(
        '-f',
        '--fps',
        help='FPS of the TAS',
        type=int,
        default=60
    )
    return parser.parse_args()


def interpolate_scale(input_scale):
    scale = input_scale.copy()

    for idx in range(len(scale)):
        if idx % 2 == 0 and idx > 1:
            scale[idx], scale[idx - 1] = scale[idx - 1], scale[idx]

    return scale


def main():
    args = _get_args()

    inputs = set()
    counts = {}

    with open(args.input_file) as f:
        for line in f:
            inputs_on_frame = [i for i in line.split('|')[1].split(':') if i != '']

            for input_str in inputs_on_frame:
                inputs.add(input_str)
                counts[input_str] = counts.get(input_str, 0) + 1

    inputs = sorted(inputs, key=lambda x: counts[x], reverse=True)
    scale = generate_scale(transpose=args.transpose, minor=args.minor)[:len(inputs)]

    if args.interpolate:
        scale = interpolate_scale(scale)

    print(json.dumps({
        'validInputs': inputs,
        'midiNotes': scale,
        'fps': args.fps,
    }))



from mido import Message, MidiFile, MidiTrack


class Track:
    def __init__(self, file_name, config, program):
        self.file_name = file_name
        self.output = MidiFile()
        self.track = MidiTrack()
        self.output.tracks.append(self.track)

        self.input_to_note = dict(zip(config['validInputs'], config['midiNotes']))
        self.fps = config['fps']

        self.track.append(Message('program_change', program=program, time=0))

        self.activated_notes = dict([(i, False) for i in config['validInputs']])
        self.beats_passed = 0

    def save(self):
        self.output.save(self.file_name)

    def handle(self, line):
        inputs = [i for i in line.split('|')[1].split(':') if i != '']

        self.disable_notes_not_in_input(inputs)

        for input_str in inputs:
            self.enable_note(input_str)

        self.beats_passed += 1

    def disable_notes_not_in_input(self, inputs):
        for input_name, activated in self.activated_notes.items():
            if activated and input_name not in inputs:
                self.activated_notes[input_name] = False
                self.track.append(
                    Message(
                        'note_off',
                        note=self.input_to_note[input_name],
                        time=self.delta,
                    ),
                )
                self.beats_passed = 0

    def enable_note(self, input_str):
        if self.activated_notes[input_str]:
            return

        self.activated_notes[input_str] = True
        self.track.append(
            Message(
                'note_on',
                note=self.input_to_note[input_str],
                velocity=64,
                time=self.delta,
            ),
        )
        self.beats_passed = 0

    @property
    def delta(self):
        return (self.beats_passed * self.output.ticks_per_beat) // (self.fps // 2)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tasmidi']

package_data = \
{'': ['*']}

install_requires = \
['mido>=1.2.9,<2.0.0']

entry_points = \
{'console_scripts': ['tasmidi-convert = tasmidi.convert:main',
                     'tasmidi-map = tasmidi.map:main']}

setup_kwargs = {
    'name': 'tasmidi',
    'version': '0.1.1',
    'description': '',
    'long_description': '# tasmidi\n\n## What is this?\n\n`tasmidi` is a collection of tools used to convert a LibTAS input file into a MIDI file. There are two steps to this: producing a mapping file to tell the program what notes correspond to what input, and using that mapping file to create the MIDI.\n\n## Installation\n\n```bash\npip install tasmidi\n```\n\n## How do I get the input file?\n\nAssuming that your LibTAS movie file is called `tas-movie.ltm`:\n\n```bash\ntar xvzf tas-movie.ltm\n```\n\nThis will produce several files, but the only one that we are interested in is called `inputs`.\n\n## Usage\n\nBoth commands in this package have their own `--help` flag for specific usage. A typical workflow looks like:\n\n```bash\ntasmidi-map inputs > mapping.json # Generate a mapping file for the inputs\ntasmidi-convert inputs -m mapping.json # Use that mapping and the input file to create a MIDI\n```\n',
    'author': 'Mark Rawls',
    'author_email': 'markrawls96@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/markrawls/tasmidi',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



root = 48  # C3

major_pattern = [2, 2, 1, 2, 2, 2, 1]
minor_pattern = [2, 1, 2, 2, 1, 2, 2]


def generate_scale(transpose=0, minor=False):
    scale = [root + transpose]
    step_pattern = minor_pattern if minor else major_pattern

    # Get us a few octaves of range
    for _ in range(3):
        for interval in step_pattern:
            scale.append(scale[-1] + interval)

    return scale
